#include <stevens.h>

 /* Creation d'une socket pour le client (socket)
  * de type stream/unix et connexion au serveur 
  */

#  define   CLIENT_PERM S_IRWXU              /* rwx pour utilisateur seulement */

int cli_conn (const char *server_path, const char*client_path) {
     /* retourne le fd de la socket du client si OK, -1 si erreur */

   int fd;                          /* fd de la socket */
   struct sockaddr_un unix_addr;    /* adresse de socket (serveur ou client) */
                                    /* cf la fin de ce fichier pour la dfinition 
                                       de struct sockaddr_un */
   int len;                         /* longueur de l'adresse de socket Unix */

      /* creation d'une socket de domaine Unix de type stream */
   if ( (fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) return(-1);

      /* construire l'adresse de la socket du client et sa longueur */
   memset(&unix_addr, 0, sizeof(unix_addr)); /* mise  0 de tous les champs */
   unix_addr.sun_family = AF_UNIX;           /* type d'adresse */
   strcpy(unix_addr.sun_path, client_path);  /* copie du nom de chemin de la socket */
   len = sizeof(unix_addr.sun_family) + strlen(unix_addr.sun_path) ;

      /* effacement du nom, s'il existe avant le binding */
   if ((unlink(client_path) <0) && (errno != ENOENT)){ /* chec */
      return(-2);
   }

       /* attacher la socket dans la hirarchie Unix */
   if (bind(fd, (struct sockaddr *) &unix_addr, len) < 0) return(-3);

      /* n'autoriser un accs  la socket qu'au propritaire du client */
   if (chmod(unix_addr.sun_path, CLIENT_PERM) < 0) return(-4);

      /* construire l'adresse de la socket du serveur et de sa longueur */
   memset(&unix_addr, 0, sizeof(unix_addr)); /* mise  0 de tous les champs */
   unix_addr.sun_family = AF_UNIX;           /* type d'adresse */
   strcpy(unix_addr.sun_path, server_path);         /* recopie du nom de chemin */
   len = sizeof(unix_addr.sun_family) + strlen(unix_addr.sun_path) ;

      /* se connecter  la socket du serveur et attendre le RdV */
   if (connect(fd, (struct sockaddr *) &unix_addr, len) < 0) return(-5);

      /* ne pas effacer ici le nom de la socket du client : 
         le serveur l'utilise peut-tre encore ... */

      /* retourner le fd de la socket du client */
   return(fd);
}


/* Pour comprendre le code, voici la structure dcrivant
 * l'adresse d'une socket AF_LOCAL (aka AF_UNIX) (#include <sys/un.h>)
 */

// struct sockaddr_un {
//    unsigned short int   sun_family;       /* type de famille (2 octets) */
//    char                 sun_path[108];    /* Path name.  */
//  };
