/* utilitaire de rception d'un fd ou d'un message d'erreur avec code
 * d'erreur, selon le protocole de Stevens (send_fd/send_err/recv_fd)
 * version avec socket et struct msghdr.msg_control (ancillary data) 
 * BSD 4.3+/4.4 / LINUX 
 */

#include <stevens.h>

static struct cmsghdr   *cmptr = NULL;    /* allocation au premier appel de recfd */

#define  CONTROLLEN  (sizeof(struct cmsghdr) + sizeof(int))
       /* taille du tampon pour recevoir un fd */

int recv_fd(int sockfd, ssize_t (*userfunc)(int, const void *, size_t)) {
  /* lit avec recvmsg sur la socket fd :
   * soit un fd tel que  0 <= fd <= 255 , 
   * soit un code d'erreur cerr ngatif tel que -255 <= cerr <= -1 
   * suivi ventuellement d'un message d'erreur messerr termin par '\n'.
   * messerr sera trait par un appel 
   * (*userfunc)(STDERR_FILENO, messerr, strlen(messerr))
   * valeur retourne : fd (0 <= fd <= 255) ou cerr (-255 <= cerr <= -1)
   */

   struct msghdr  msg;            /* message lu par recvmsg */
   struct iovec   iov[1];         /* tableau d'un seul granule pour E/S clates */
   int            newfd;          /* le fd reu */
   int            messerr_len;    /* longueur effective du message d'erreur (sur LINUX != mesg_size)*/
   int            mesg_size;      /* nb de caractres lus */
   int            status;         /* code d'erreur lu */
   char           buf[MAXLINE];   /* tampon de rception du message d'erreur */
   char           *ptr;           /* ptr de parcours d'analyse du message reu */

   /* allocation d'un tampon pour recevoir un en-tte de control et un entier */
   if (cmptr == NULL && (cmptr = malloc(CONTROLLEN)) == NULL) return(-1);

   status = -1;
   while ( status <0 ) { /* tant qu'il y a qqchose  lire sur la socket, lire */
   
	  /* prparer la lecture d'un message */
      iov[0].iov_base = buf;
      iov[0].iov_len  = MAXLINE;
      msg.msg_iov     = iov;
      msg.msg_iovlen  = 1;
      msg.msg_name    = NULL;
      msg.msg_namelen = 0;
      msg.msg_control    = (caddr_t) cmptr;
      msg.msg_controllen = CONTROLLEN;

		/* lecture du message */
      if ( (mesg_size = recvmsg(sockfd, &msg, 0)) < 0)
        errno_exit("passing_fd.rcv_fd.T1: erreur de lecture du message");
      else if (mesg_size == 0) {
         errno_return("passing_fd.rcv_fd.T2: connexion ferme par le serveur");
         return(-1);
      }

      /* On a lu un message, analysons le protocole pour distinguer fd ou messerr 
       * syntaxe du message (fait au moins 2 caractres) :
       * [ message sans nul termin par \n ] nul ( nul | coderreur)
       * nul nul => un fd se trouve dans la partie controle du message
       * nul code-erreur-non-nul => la valeur absolue du code d'erreur
       */

      for (ptr = buf; ptr < &buf[mesg_size]; ) { 
         /* lecture du message complet */
		  if (*ptr++ == 0) { /* on a trouv le premier nul */
#ifndef LINUX
			 /* ce contrle peut chouer sous LINUX sans qu'il y ait un pb: on le supprime */
         if (ptr != &buf[mesg_size-1]) /* garbage en fin de message */
			   err_exit("passing_fd.rcv_fd.T3: "
			            "format de message erron (e.g. non termin par '\\n')");
#endif
			 /* le car suivant le nul, point par ptr doit contenir le status */
			 status = *ptr & 255; /* mettre l'octet du status dans l'entier status */
            if (status == 0) { /* fd dans la partie contrle du message */
               if (msg.msg_controllen != CONTROLLEN)
                  errno_exit("passing_fd.rcv_fd.T4: "
                    "mauvaise longueur de la structure de contrle pour le passage fd");
               newfd = *(int *)CMSG_DATA(cmptr); /* extraction du fd par la macro */
            } else { /* status indique le code d'erreur */
               newfd = -status;
            }
            mesg_size -= 2; /* maj de mesg_size */
         }/* if (*ptr++ == 0) */
      }/* for (ptr = buf; ptr < &buf[mesg_size]; ) */

      if (mesg_size > 0) { /* On a reu un message d'erreur  traiter */
		  if ((*userfunc)(STDERR_FILENO, buf, strlen(buf)) != strlen(buf)) {
           errno_return("passing_fd.rcv_fd.T6: "
                    "errreur d'criture du message d'erreur sur stderr");
            return(-1);
        }
      }
   } /* while ( status <0 ) ...  tant qu'il y a qqchose  lire sur la socket, lire */

   return(newfd); /* fd ou -status */
}
