/* utilitaire d'envoi d'un fd selon le protocole de Stevens
 * send_fd/send_err/recv_fd
 * version avec socket et struct msghdr.msg_control (ancillary data) 
 * BSD 4.3+/4.4 / LINUX 
 */

#include	<stevens.h>

static struct cmsghdr	*cmptr = NULL;	 /* allocation au premier appel de sendfd */

#define	CONTROLLEN	(sizeof(struct cmsghdr) + sizeof(int))
       /* taille du tampon pour recevoir un fd */


int send_fd(int clifd, int fd) {
/* passe  travers la socket (de type STREAM/UNIX)
 * un descripteur de fichier (fd)  un autre processus (client).
 * Si fd<0, alors |fd| est interprt comme un code d'erreur 
 * La transmission utilise sendmsg(2) avec un champ de contrle.
 * la syntaxe du protocole est la suivante (2 octets envoys comme donne) :
 * nul nul => le fd se trouve dans la partie contrle du message
 * nul /= 0 => il n'y a pas de fd de transmis, mais le code d'erreur |fd|
 * (l'intrt du premier nul est de terminer le message d'erreur envoy juste avant
 * par send_err).
 * code de retour : 0 si OK, -1 si erreur
 */
	struct msghdr	msg;
	struct iovec	iov[1];
	char		   	buf[2];	/* 2 octets de donne pour le protocole */

	  /* prparer l'envoi du message */
	iov[0].iov_base = buf;
	iov[0].iov_len  = 2;
	msg.msg_iov     = iov;
	msg.msg_iovlen  = 1;
	msg.msg_name    = NULL;
	msg.msg_namelen = 0;
	buf[0] = 0;			/* le premier octet de donne est nul (cf protocole) */

	if (fd < 0) { 
	  /* code d'erreur  envoyer, pas de champ de contrle  */
		msg.msg_control    = NULL;
		msg.msg_controllen = 0;
		buf[1] = -fd;	/* |fd| >0 => code d'erreur */
		if (buf[1] == 0) ERR_EXIT; /* possible si |fd| >= 256 */

	} else { /* fd  transmettre dans le champ msg.msg_control */
	  /* allocation du champ de contrle si ncessaire */
		if (cmptr == NULL && (cmptr = malloc(CONTROLLEN)) == NULL) return(-1);
      /* indiquer qu'il s'agit d'un passage de fd */
		cmptr->cmsg_level  = SOL_SOCKET;
		cmptr->cmsg_type   = SCM_RIGHTS;
		cmptr->cmsg_len    = CONTROLLEN;
		msg.msg_control    = (caddr_t) cmptr;
		msg.msg_controllen = CONTROLLEN;
      /* passer le fd avec un macro */
		*(int *)CMSG_DATA(cmptr) = fd;		/* the fd to pass */
		buf[1] = 0;		/* cet octet doit tre nul pour indiquer  recv_fd que c'est un fd */
	}

   /* envoyer le message au client */
	if (sendmsg(clifd, &msg, 0) != 2) return(-1);
	return(0);
}
