#include <stevens.h>

/* Attente qu'un client se manifeste sur la socket du serveur.
 * A ce moment, retourne le UID du client obtenu par le pathname
 * de la socket du client et le fd d'accs  cette socket.
 */

#define  STALE 30 
    /* la cration du fifo du client doit avoir t faite depuis moins de STALE sec.
     * cela limite les possibilits de piratage... */

int serv_accept (int server_fd, uid_t *uidptr) {
   /* retourne le fd de la socket du client si OK, -1 si erreur
    * retourne dans uidptr* la valeur du uid du client
    */

   int                  client_fd;
   struct sockaddr_un   unix_addr;   /* adresse de socket du client */
                                     /* cf la fin de ce fichier pour la dfinition 
                                        de struct sockaddr_un */
   int                  len;         /* longueur de l'adresse de socket Unix */
   struct stat          statbuf;
   time_t               staletime;

   /* attendre qu'un client se connecte sur la socket du serveur
    * accept rend dans unix_addr l'adresse de la socket du client
    * et dans len la longueur de la chane <unix_addr.sun_family><unix_addr.sun_path>
    * <unix_addr.sun_path> est <= 108, cf  la fin de fichier
    */
   len = sizeof(unix_addr);
   if ( (client_fd = accept(server_fd, (struct sockaddr *) &unix_addr, &len)) < 0)
      return(-1);    /* souvent errno=EINTR, si un signal a t reu */

   /* mettre un '\0' de scurit  la fin du nom de chemin 
      de la socket du client */
	len -= sizeof(unix_addr.sun_family); /* length of unix_addr.sun_path */
	unix_addr.sun_path[ len ] = 0 ;

   /* demander les attributs de la socket du client */
   if (stat(unix_addr.sun_path, &statbuf) < 0) {
        return(-2);
   }
   if (S_ISSOCK(statbuf.st_mode) == 0) return(-3); /* pas une socket */
   if ((statbuf.st_mode & (S_IRWXG | S_IRWXO)) ||
      (statbuf.st_mode & S_IRWXU) != S_IRWXU)
        return(-4);  /* les protections ne sont pas rwx------ */

   /* si l'ge de la socket du client est suprieur  STALE, elle est trop vieille */
   staletime = time(NULL) - STALE;
   if (statbuf.st_atime < staletime || statbuf.st_ctime < staletime ||
      statbuf.st_mtime < staletime)  return(-5);

   /* placer dans uidptr* le uid du client */
   if (uidptr != NULL)
      *uidptr = statbuf.st_uid;

   return(client_fd);
}

/* Pour comprendre le code, voici la structure dcrivant
 * l'adresse d'une socket AF_LOCAL (aka AF_UNIX) (#include <sys/un.h>)
 */

// struct sockaddr_un {
//    unsigned short int   sun_family;       /* type de famille (2 octets) */
//    char                 sun_path[108];    /* Path name.  */
//  };
