/* En-tte de Stevens largement revisit par Roger Rousseau, <rr@unice.fr> */

#ifndef  _stevens_h
#define  _stevens_h

/* Inclusion de la plupart des en-ttes de primitives systme, 
 * pour simplifier la programmation
 * (les ordinateurs sont aujourd'hui si rapides, pourquoi s'embter...)
*/

#include <sys/types.h>  /* required for some of our prototypes */
#include <stdio.h>      /* for convenience */
#include <stdlib.h>     /* for convenience */
#include <string.h>     /* for convenience */
#include <unistd.h>     /* for convenience */
#include <sys/stat.h>   /* pour les primitives d'introspection de fichiers */
#include <sys/fcntl.h>  /* pour open ... */
#include <dirent.h>     /* lecture d'un rpertoire */
#include <limits.h>     /* limites systme */
#include <time.h>       /* dcodage du temps */

#ifdef CYGWIN
#   include <sys/uio.h>
#   include <sys/timeb.h>
#endif

#ifdef LINUX
#   include <sys/time.h>       /* dcodage du temps */
#   include <sys/timeb.h>
#   include <sys/ioctl.h>
#   include <stropts.h>
#endif

#include <pwd.h>        /* pour avoir accs aux champs de /etc/passwd */
#include <grp.h>        /* pour avoir accs aux champs de /etc/group */
#include <sys/wait.h>   /* pour les diffrents wait et dcodage des codes de retour */
#include <errno.h>      /* pour l'acces aux constantes d'erreurs systeme */
#include <signal.h>

#ifdef SOLARIS
#   include	<sys/int_types.h> /* dfinit uint32_t et d'autres types entier */
#endif

#ifndef DARWIN
#   include <termio.h>     /* uint32_t requis sur Solaris */
#endif

#include <syslog.h>
#include <sys/socket.h>
#include <sys/param.h>

#ifndef DARWIN
#   include <poll.h>
#endif /* DARWIN */

#include <sys/un.h>
#include	<stddef.h>
// #include	<stdarg.h>

/*********** fin des inclusions *******************/

#define  MAXLINE  4096        /* max line length */

#define  FILE_MODE   (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
               /* default file access permissions for new files */
#define  DIR_MODE (FILE_MODE | S_IXUSR | S_IXGRP | S_IXOTH)
               /* default permissions for new directories */

#define  CREAT_WRONLY_FLAGS (O_CREAT | O_WRONLY | O_TRUNC)
#define  CREAT_RDWR_FLAGS   (O_CREAT | O_RDWR   | O_TRUNC)

typedef  void  Sigfunc(int);  /* for signal handlers */

               /* 4.3BSD Reno <signal.h> doesn't define SIG_ERR */
#if   defined(SIG_IGN) && !defined(SIG_ERR)
#define  SIG_ERR  ((Sigfunc *)-1)
#endif

#define  min(a,b) ((a) < (b) ? (a) : (b))
#define  max(a,b) ((a) > (b) ? (a) : (b))

/* prototypes pour les fonctions utilitaires de Stevens */
char  *path_alloc(int *);              /* Prog: pathalloc */
int    open_max(void);                 /* Prog: openmax */
void   clr_fl(int, int);               /* Prog: setfl */
void   set_fl(int, int);               /* Prog: setfl */
void   pr_exit(int);                   /* Prog: prexit */
void   pr_mask(const char *);          /* Prog: prmask */
Sigfunc  *signal_intr(int, Sigfunc *); /* Prog: signal_intr_function */
int       tty_cbreak(int);             /* Prog: raw */
int       tty_raw(int);                /* Prog: raw */
int       tty_reset(int);              /* Prog: raw */
void   tty_atexit(void);               /* Prog: raw */

#ifdef LINUX
#ifdef TERMIOS_H
struct termios *tty_termios(void);     /* Prog: raw */
#endif
#else
#ifdef   ECHO                          /* only if <termios.h> has been included */
struct termios *tty_termios(void);     /* Prog: raw */
#endif
#endif

void                                   sleep_us(unsigned int);   /* Ex: sleepus */
ssize_t   readn(int, void *, size_t);         /* Prog: readn */
ssize_t   writen(int, const void *, size_t);  /* Prog: writen */
int       daemon_init(void);                  /* Prog: daemoninit */
int       s_pipe(int *);                      /* Progs: plusieurs versions de s_pipe */
int       recv_fd(int, ssize_t (*func)(int, const void *, size_t));
                                             /* Progs: plusieurs versions de recvfd */
int       send_fd(int, int);                 /* Progs: plusieurs versions de sendfd */
int       send_err(int, int, const char *);  /* Prog:  senderr */
int       serv_listen(const char *);         /* Progs: plusieurs versions de servlisten */
int       serv_accept(int, uid_t *);         /* Progs: plusieurs versions de servaccept */
int       cli_conn(const char *, const char*); /* Progs: plusieurs versions de cliconn */
int       buf_args(char *, int (*func)(int, char **)); /* Prog: bufargs */
int       ptym_open(char *);                 /* Progs: plusieurs versions de ptyopen */
int       ptys_open(int, char *);            /* Progs: plusieurs versions de ptyopen */
#ifdef   TIOCGWINSZ
pid_t  pty_fork(int *, char *, const struct termios *,
              const struct winsize *);       /* Prog: ptyfork */
#endif
void  TELL_WAIT(void);                       /* Prog: tellwait.c (deux versions possibles */
void  TELL_PARENT(pid_t);                    /* avec signaux ou pipes) */ 
void  TELL_CHILD(pid_t);                     /* (cf Stevens race synchronizations) */
void  WAIT_PARENT(void);
void  WAIT_CHILD(void);

int      lock_reg(int, int, int, off_t, int, off_t); /* Prog: lockreg */
#define  read_lock(fd, offset, whence, len) \
         lock_reg(fd, F_SETLK, F_RDLCK, offset, whence, len)
#define  readw_lock(fd, offset, whence, len) \
         lock_reg(fd, F_SETLKW, F_RDLCK, offset, whence, len)
#define  write_lock(fd, offset, whence, len) \
         lock_reg(fd, F_SETLK, F_WRLCK, offset, whence, len)
#define  writew_lock(fd, offset, whence, len) \
         lock_reg(fd, F_SETLKW, F_WRLCK, offset, whence, len)
#define  un_lock(fd, offset, whence, len) \
         lock_reg(fd, F_SETLK, F_UNLCK, offset, whence, len)

pid_t lock_test(int, int, off_t, int, off_t);        /* Prog: locktest */

#define  is_readlock(fd, offset, whence, len) \
         lock_test(fd, F_RDLCK, offset, whence, len)
#define  is_writelock(fd, offset, whence, len) \
         lock_test(fd, F_WRLCK, offset, whence, len)

/* Routines et macros de traitement des erreurs (ide de Stevens, renommage de RR)
 * Correspondances des procedures d'erreur :
 * Stevens   => Rousseau      => MACROS RR
 * err_ret   => errno_return  => ERRNO_RETURN
 * err_sys   => errno_exit    => ERRNO_EXIT
 * err_dump  => errno_dump    => ERRNO_DUMP
 * err_msg   => err_return    => ERR_RETURN
 * err_quit  => err_exit      => ERR_EXIT
 * logno_msg => logno_return
 * logno_sys => logno_exit
 * log_msg   => log_return
 * log_sys   => log_exit
 */

/* Primitives de Stevens (renommes) pour les erreurs ... */

void errno_return(const char *fmt, ...);
void errno_exit(const char *fmt, ...);
void errno_dump(const char *fmt, ...);
void err_return(const char *fmt, ...);
void err_exit(const char *fmt, ...);

void log_open(int debug, const char *ident, int option, int facility);
void logno_return(const char *fmt, ...);
void logno_exit(const char *fmt, ...);
void log_return(const char *fmt, ...);
void log_exit(const char *fmt, ...);

/* Macros de Roger Rousseau pour les erreurs ... */

#define ERRNO_RETURN (errno_return("ERRNO_RETURN: %s => problme  la ligne %d", __FILE__, __LINE__))
#define ERRNO_EXIT (errno_exit("ERRNO_EXIT: %s => erreur fatale  la ligne %d", __FILE__, __LINE__))
#define ERRNO_DUMP (errno_dump("ERRNO_DUMP: %s => erreur fatale  la ligne %d ; dump...",\
                     __FILE__, __LINE__))
#define ERR_RETURN (err_return("ERR_RETURN: %s => problme  la ligne %d", __FILE__, __LINE__))
#define ERR_EXIT (err_exit("ERR_EXIT: %s => erreur fatale  la ligne %d", __FILE__, __LINE__))

/* Utilitaires de Roger Rousseau (rr@unice.fr) */
char* get_login_name(uid_t uid) ;                                      /* Prog: get_login_name */
int fetch_words (char *line, int nwords_max, char *word1, ...);      /* Prog: fetch_words */

#endif   /* _stevens_h */
